﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                            module_assassination.lua
--
--	Copyright (C) 2012 Deronda
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.  Some of the code was pulled
--  from existing clcInfo modules and the example module.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      Deronda (@ Stormreaver)
------------------------------------------------------------------------------
-- Description:
--   Actions module for assassination rogues
------------------------------------------------------------------------------
if not clcInfo then return end       -- need clcInfo to run
if not clcInfo_Rogue then return end -- need clcInfo_Rogue
local _, englishClass = UnitClass("player") -- need to be a rogue
if englishClass ~= clcInfo_Rogue.CLASS then return end

-- Get local copies of functions/tables.  This will speed up execution.
local clcInfo_Rogue = clcInfo_Rogue
local spells = clcInfo_Rogue.spells
local GetTimeLeft = clcInfo_Rogue.GetTimeLeft
local GetCooldown = clcInfo_Rogue.GetCooldown
local IconNext = clcInfo_Rogue.IconNext
local IconCurrent = clcInfo_Rogue.IconCurrent
local BleedDebuffLeftOnTarget = clcInfo_Rogue.BleedDebuffLeftOnTarget
local CalculateTargetHealthPct = clcInfo_Rogue.CalculateTargetHealthPct
local UnitPower = UnitPower
local UnitAura = UnitAura
local GetNumPartyMembers = GetNumPartyMembers
local GetNumRaidMembers = GetNumRaidMembers

-- Assign module variables
local specname = "assassination"
local prio_arg = "assassinationprio"
local sample_prio = "tricks garrote snd rupture vendetta cb envenom backstab mutilate vanish"
local prio = "garrote snd rupture envenom backstab mutilate"
local class_module
local class_module_name

local db_default = {
    char_var = false,
}
local db

local tdb_default = {
    version = clcInfo_Rogue.VERSION,
    prio = prio,
    ignore_rupture_no_bleed = false,
    rangePerSkill = false,
}
local tdb

-- actions -------------------------------------------------------------------------
-- local copies of spell names and ids to ugly up code but speed action functions up
local ColdBlood = spells["Cold Blood"].name
local Envenom = spells["Envenom"].name
local Overkill = spells["Overkill"].name
local Rupture = spells["Rupture"].name
local SliceAndDice = spells["Slice and Dice"].name
local Stealth = spells["Stealth"].name
local Vanish = spells["Vanish"].name
local FuryoftheDestroyer = spells["Fury of the Destroyer"].name

local BackstabId = spells["Backstab"].id
local ColdBloodId = spells["Cold Blood"].id
local EnvenomId = spells["Envenom"].id
local GarroteId = spells["Garrote"].id
local MutilateId = spells["Mutilate"].id
local RuptureId = spells["Rupture"].id
local SliceAndDiceId = spells["Slice and Dice"].id
local TricksId = spells["Tricks of the Trade"].id
local VanishId = spells["Vanish"].id
local VendettaId = spells["Vendetta"].id

local actions  = {
	tricks = {
		id = TricksId,
		GetCD = function(cur_id)
            if cur_id ~= TricksId and (GetNumRaidMembers() > 0 or GetNumPartyMembers() > 0) then
                return GetCooldown(TricksId)
            end
            return 100
        end,
		UpdateStatus = function(combo_points)
            return combo_points
		end,
		info = spells["Tricks of the Trade"].name,
	},
	garrote = { -- don't bother to use IsStealthed since this is the only time we check it
		id = GarroteId,
		GetCD = function(cur_id)
	        if cur_id ~= GarroteId 
               and (((UnitAura("player", Stealth)) or (UnitAura("player", Vanish)))
                   or cur_id == VanishId)
            then
                return GetCooldown(GarroteId)
            end
            return 100
		end,
		UpdateStatus = function(combo_points)
            return combo_points + 1
		end,
		info = spells["Garrote"].name,
	},
    snd = {
		id = SliceAndDiceId,
		GetCD = function(cur_id, combo_points)
	        if cur_id ~= SliceAndDiceId 
               and combo_points > 0
               and not (UnitAura("player", SliceAndDice))
            then
                return GetCooldown(SliceAndDiceId)
            end
            return 100
		end,
		UpdateStatus = function()
            if GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL") > 0 then
                return 5
            end
            return 0
		end,
		info = spells["Slice and Dice"].name,
	},
    rupture = {
		id = RuptureId,
		GetCD = function(cur_id, combo_points)
            if cur_id ~= RuptureId
               and combo_points > 1 
               and GetTimeLeft("target", Rupture, "HARMFUL|PLAYER") <= 1
               and GetTimeLeft("player", SliceAndDice, "HELPFUL") > 6
               and (not tdb.ignore_rupture_no_bleed or BleedDebuffLeftOnTarget() > 0)
            then
                return GetCooldown(RuptureId)
            end
            return 100
		end,
		UpdateStatus = function()
            if GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL") > 0 then
                return 5
            end
            return 0
		end,
		info = spells["Rupture"].name,
	},
    vendetta = {
        id = VendettaId,
		GetCD = function(cur_id)
            if cur_id ~= VendettaId then
                return GetCooldown(VendettaId)
            end
            return 100
		end,
		UpdateStatus = function(combo_points)
            return combo_points
		end,
		info = spells["Vendetta"].name,
	},
	cb = { -- if next attack is an envenom (check cp & target health and rupture cd to decide)
		id = ColdBloodId,
		GetCD = function(cur_id, combo_points)
            if cur_id ~= ColdBloodId then
                local target_health = CalculateTargetHealthPct()
                if GetTimeLeft("target", Rupture, "HARMFUL|PLAYER") > 4
                   and not (UnitAura("player", ColdBlood))
                   and ((target_health > 34 and combo_points > 3) or (target_health < 35 and combo_points > 4))
                then
                    return GetCooldown(ColdBloodId)
                end
            end
            return 100
		end,
		UpdateStatus = function(combo_points)
			return combo_points, true -- true = ignore time increment for next ability choice
		end,
		info = spells["Cold Blood"].name,
	},
	envenom = {
		id = EnvenomId,
		GetCD = function(_, combo_points)
            local envenom_cd = GetCooldown(EnvenomId)
            if combo_points > 1 and GetTimeLeft("player", SliceAndDice, "HELPFUL") < 3 then
                return envenom_cd
            end
            local target_health = CalculateTargetHealthPct()
            if (target_health > 34 and combo_points > 3) or (target_health < 35 and combo_points > 4) then
                local envenom_left = GetTimeLeft("player", Envenom, "PLAYER")
                local yellow
                if UnitPower("player") > 90 or GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL") > 0
                then
                    return envenom_cd
                elseif envenom_left > envenom_cd then
                    envenom_cd = envenom_left
                    yellow = true -- yellow icon so user knows to wait
                end
                return envenom_cd, yellow
            end
            return 100
		end,
		UpdateStatus = function()
            if GetTimeLeft("player", FuryoftheDestroyer, "HELPFUL") > 0 then
                return 5
            end
            return 0
		end,
		info = spells["Envenom"].name,
	},
    backstab = {
		id = BackstabId,
		GetCD = function(_, combo_points)
            if combo_points < 5 and CalculateTargetHealthPct() < 35 then
                return GetCooldown(BackstabId)
            end
            return 100
		end,
		UpdateStatus = function(combo_points)
            return combo_points + 1
		end,
		info = spells["Backstab"].name,
	},    
 	mutilate = {
		id = MutilateId,
		GetCD = function(_, combo_points)
            if combo_points < 4 and CalculateTargetHealthPct() >= 35 then
                return GetCooldown(MutilateId)
            end
            return 100
		end,
		UpdateStatus = function(combo_points)
            return combo_points + 2
		end,
		info = spells["Mutilate"].name,
	},
	vanish = {
		id = VanishId,
		GetCD = function(cur_id, combo_points)
            if cur_id ~= VanishId and combo_points < 5 and GetTimeLeft("player", Overkill) < 1 then
                local yellow = (UnitPower("player") < 45) -- need enough energy to immediately garrote
                return GetCooldown(VanishId), yellow
            end
            return 100
		end,
		UpdateStatus = function(combo_points)
			return combo_points, true -- true = ignore time increment since not on gcd
		end,
		info = spells["Vanish"].name,
	},
}

----FUNCTIONS-------------------------------------------------------------------
-- The following are called once or infrequently, so speed isn't an issue.

-- called whenever module options are loaded
local GetDB
local SetDB
local GetTDB
local SetTDB

--initialize module.
class_module, class_module_name = clcInfo_Rogue:init_module(specname)

--called when template updates.
function class_module:OnTemplatesUpdate()
	tdb, GetTDB, SetTDB = clcInfo_Rogue:OnTemplatesUpdate(class_module_name, actions, tdb_default, specname)
end

-- called each time clcInfo initializes
function class_module.OnInitialize()
	db, GetDB, SetDB = clcInfo_Rogue:OnInitialize(class_module_name, db_default)
end

-- called when module options are opened
local function LoadAssassinationModule()
    local _, talent_tree_name = GetTalentTabInfo(1)
    clcInfo_Rogue.LoadModule(class_module_name, talent_tree_name, prio_arg, sample_prio,
                             actions, GetDB, SetDB, GetTDB, SetTDB)
end
clcInfo.optionsCMLoaders[#(clcInfo.optionsCMLoaders) + 1] = LoadAssassinationModule

-- giving queue as command line
local function CmdAssassination(args)
	clcInfo_Rogue:CmdLine(tdb, args, actions, specname)
end

clcInfo.cmdList[prio_arg] = CmdAssassination

----ABILITY ICONS--------------------------------------------------------------
----Add these to the clcInfo.env (so they live with IconAura, etc.
local emod = clcInfo.env
function emod.IconAssassinationCurrent(...)
	return IconCurrent(actions, specname, tdb.rangePerSkill)
end
function emod.IconAssassinationNext(...)
	return IconNext()
end